// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ButtonImg.cpp
//
// $Id: ButtonImg.cpp,v 1.2 2002/02/26 06:59:40 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "ButtonImg.h"

#include "ErrorManager.h"
#include "String.h"

//------------------------------------------------------------------------------

namespace xb {

//------------------------------------------------------------------------------

ButtonImg::ButtonImg(const std::string& strNormalFile,
					 const std::string& strOverFile,
					 const std::string& strDownFile) :
	m_pPixNormal(NULL),
	m_pPixOver(NULL),
	m_pPixDown(NULL)
{
	SetFiles(strNormalFile, strOverFile, strDownFile);
	
	m_pixButton.set(m_pPixNormal->pix(), m_pPixNormal->bit());
	
	add(m_pixButton);

	button_press_event.connect(SigC::slot(this, &ButtonImg::ButtonDown));
	button_release_event.connect(SigC::slot(this, &ButtonImg::ButtonUp));

	enter_notify_event.connect(SigC::slot(this, &ButtonImg::ButtonEnter));
	leave_notify_event.connect(SigC::slot(this, &ButtonImg::ButtonLeave));
	
	// set_usize(m_pixNormal.width(), m_pixNormal.height());
	
	shape_combine_mask(m_pPixNormal->bit(), 0, 0);
}

//------------------------------------------------------------------------------

ButtonImg::~ButtonImg()
{

}

//------------------------------------------------------------------------------

void
ButtonImg::SetFiles(const std::string& strNormalFile,
					const std::string& strOverFile,
					const std::string& strDownFile)
{
	// PATRICK: can't delete these since they are still being displayed?

	delete m_pPixNormal;
	delete m_pPixOver;
	delete m_pPixDown;

	m_pPixNormal = new Gtk::ImageLoader(strNormalFile);
	m_pPixOver = new Gtk::ImageLoader(strOverFile);
	m_pPixDown = new Gtk::ImageLoader(strDownFile);

	AssertNotNull(m_pPixNormal);
	AssertNotNull(m_pPixOver);
	AssertNotNull(m_pPixDown);

	m_pixButton.set(m_pPixNormal->pix(), m_pPixNormal->bit());
	shape_combine_mask(m_pPixNormal->bit(), 0, 0);

}
	
//------------------------------------------------------------------------------

int
ButtonImg::ButtonDown(GdkEventButton* pButton)
{
	m_pixButton.set(m_pPixDown->pix(), m_pPixDown->bit());
	show_all();

	return 0;
}

//------------------------------------------------------------------------------

int
ButtonImg::ButtonUp(GdkEventButton* pButton)
{
	m_pixButton.set(m_pPixNormal->pix(), m_pPixNormal->bit());
	show_all();
	clicked();

	return 0;
}

//------------------------------------------------------------------------------

int
ButtonImg::ButtonEnter(GdkEventCrossing* pEvent)
{
	m_pixButton.set(m_pPixOver->pix(), m_pPixOver->bit());
	show_all();
	
	return 0;
}

//------------------------------------------------------------------------------

int
ButtonImg::ButtonLeave(GdkEventCrossing* pEvent)
{
	m_pixButton.set(m_pPixNormal->pix(), m_pPixNormal->bit());
	show_all();

	return 0;
}

//------------------------------------------------------------------------------

void
ButtonImg::Dump()
{
	EM::I()->Trace() << "pixbutton width:  " << m_pixButton.width() << endl;
	EM::I()->Trace() << "this width:  " << width() << endl;
	EM::I()->Trace() << "pixmap width:  " << m_pPixNormal->pix().width() << endl;
	EM::I()->Trace() << "pixmap height:  " << m_pPixNormal->pix().height() <<
		endl;
	EM::I()->Trace() << "pixmap x:  " << m_pPixNormal->pix().x() << endl;
	EM::I()->Trace() << "pixmap y:  " << m_pPixNormal->pix().y() << endl;
}

//------------------------------------------------------------------------------

};

//------------------------------------------------------------------------------
