// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ButtonImgToggle.cpp
//
// $Id: ButtonImgToggle.cpp,v 1.2 2002/02/26 06:59:40 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "ButtonImgToggle.h"

#include "ErrorManager.h"
#include "String.h"

//------------------------------------------------------------------------------

namespace xb {

//------------------------------------------------------------------------------

ButtonImgToggle::ButtonImgToggle(const std::string& strNormalFile,
					 const std::string& strOverFile,
					 const std::string& strDownFile) :
	m_pixNormal(strNormalFile),
	m_pixOver(strOverFile),
	m_pixDown(strDownFile),
	m_bToggled(false)
{
	m_pixButton.set(m_pixNormal.pix(), m_pixNormal.bit());
	
	add(m_pixButton);

	button_press_event.connect(SigC::slot(this, &ButtonImgToggle::ButtonDown));
	// button_release_event.connect(SigC::slot(this, &ButtonImgToggle::ButtonUp));

	enter_notify_event.connect(SigC::slot(this, &ButtonImgToggle::ButtonEnter));
	leave_notify_event.connect(SigC::slot(this, &ButtonImgToggle::ButtonLeave));
	
	// set_usize(m_pixNormal.width(), m_pixNormal.height());
	
	shape_combine_mask(m_pixNormal.bit(), 0, 0);
}

//------------------------------------------------------------------------------

ButtonImgToggle::~ButtonImgToggle()
{

}

//------------------------------------------------------------------------------

int
ButtonImgToggle::ButtonDown(GdkEventButton* pButton)
{

	if (!m_bToggled) {
		m_pixButton.set(m_pixDown.pix(), m_pixDown.bit());
		m_bToggled = true;
	}
	else {
		m_pixButton.set(m_pixNormal.pix(), m_pixNormal.bit());
		m_bToggled = false;
	}

	toggled();
	show_all();

	return 0;
}

//------------------------------------------------------------------------------

void
ButtonImgToggle::SetState(bool bToggled)
{
	m_bToggled = bToggled;
	if (m_bToggled) {
		m_pixButton.set(m_pixDown.pix(), m_pixDown.bit());
	}
	else {
		m_pixButton.set(m_pixNormal.pix(), m_pixNormal.bit());
	}
	show_all();
}

//------------------------------------------------------------------------------

/*
int
ButtonImgToggle::ButtonUp(GdkEventButton* pButton)
{
	m_pixButton.set(m_pixNormal.pix(), m_pixNormal.bit());
	show_all();
	clicked();

	return 0;
}
*/
//------------------------------------------------------------------------------

int
ButtonImgToggle::ButtonEnter(GdkEventCrossing* pEvent)
{
	if (!m_bToggled) {
		m_pixButton.set(m_pixOver.pix(), m_pixOver.bit());
		show_all();
	}
	
	return 0;
}

//------------------------------------------------------------------------------

int
ButtonImgToggle::ButtonLeave(GdkEventCrossing* pEvent)
{
	if (!m_bToggled) {
		m_pixButton.set(m_pixNormal.pix(), m_pixNormal.bit());
		show_all();
	}

	return 0;
}

//------------------------------------------------------------------------------

void
ButtonImgToggle::Dump()
{
	EM::I()->Trace() << "pixbutton width:  " << m_pixButton.width() << endl;
	EM::I()->Trace() << "this width:  " << width() << endl;
}

//------------------------------------------------------------------------------

};

//------------------------------------------------------------------------------
