// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// LogStream.cpp
//
// $Id: LogStream.cpp,v 1.1 2002/02/26 07:00:03 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "LogStream.h"

namespace xb {

//------------------------------------------------------------------------------

LogStream::LogStream() :
	m_nLogLevel(TRACE),
	m_pFileBuf(NULL),
	m_pNullStream(NULL),
	m_bFileLine(true)
{
	m_pFileBuf = new filebuf();
	m_pFileBuf->open("/dev/null", ios::out);
	m_pNullStream = new ostream(m_pFileBuf);
}

//------------------------------------------------------------------------------

LogStream::~LogStream()
{
	delete m_pNullStream;
	delete m_pFileBuf;
}

//------------------------------------------------------------------------------

ostream&
LogStream::Noise() const
{
	if (m_nLogLevel < TRACE) {
		// cout << __FILE__ << " [" << __LINE__ << "]:  ";
		return cout;
	}

	return *m_pNullStream;
}

//------------------------------------------------------------------------------

ostream&
LogStream::Trace() const
{
	if (m_nLogLevel < WARNING) {
		// cout << __FILE__ << " [" << __LINE__ << "]:  ";
		return cout;
	}

	return *m_pNullStream;
}

//------------------------------------------------------------------------------

ostream&
LogStream::Warn() const
{
	if (m_nLogLevel < ERROR) {
		cerr << "* Warning: ";
		return cerr;
	}

	return *m_pNullStream;
}

//------------------------------------------------------------------------------

ostream&
LogStream::Error() const
{
	if (m_nLogLevel < NONE) {
		cerr << "** Error: ";
		return cerr;
	}

	return *m_pNullStream;
}

//------------------------------------------------------------------------------

};

//------------------------------------------------------------------------------
