// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// EncoderLame.h
//
// $Id: EncoderLame.h,v 1.4 2002/03/12 00:01:01 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#if HAVE_CONFIG_H
#include <config.h>
#endif

#ifndef __EncoderLame__
#define __EncoderLame__


// needed for redhat users
#ifdef HAVE_LAME_LAME_H
#include <lame/lame.h>
#elif defined HAVE_LAME_H 
#include <lame.h>
#endif

#include <string>
#include <xb/Filename.h>

#include "Encoder.h"

//------------------------------------------------------------------------------

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class EncoderLame : public Encoder
{
	// Constants
	private:
		static const unsigned int MAX_U_32_NUM = 0xFFFFFFFF;
		static const int WAV_ID_WAVE =  0x57415645;
		static const int WAV_ID_FMT =   0x666d7420;
		static const int WAV_ID_DATA =  0x64617461;
		
	// Types
	// Variables
	private:
		lame_global_flags* m_pLameFlags;

		xb::Filename m_strInFilename;
		xb::Filename m_strOutFilename;

		FILE* m_pInFile;
		FILE* m_pOutFile;
		
		int m_nPCMBitWidth;
		bool m_bCountSamplesCarefully;
		unsigned int m_nNumSamplesRead;
	
	// Methods
	public:
		EncoderLame();
		~EncoderLame();

		std::string EncodeWAV(const std::string& strFilename);
		
	private:
		void Init();
		void InitFiles();
		void InitInFile();
		void InitOutFile();

		FILE* OpenSoundFile(const std::string& strFilename);
		bool ParseFileHeader(FILE* pFile);

		void LameEncode();
		int GetAudio(int anBuffer[2][1152]);
		int ReadSamplesPCM(int anSampleBuffer[2304], int nFrameSize,
						   int nSamplesToRead);
		int UnpackReadSamples(int nSamplesToRead, int nBytesPerSample,
							  int nSwapOrder, int* pSampleBuffer,
							  FILE* pPCMin);
		
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
