// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// BSBuffer.cpp
//
// $Id: BSBuffer.cpp,v 1.1 2002/01/25 05:23:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "BSBuffer.h"

#include <string.h>

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

BSBuffer::BSBuffer() :
	bsnum(0)
{
	bsbuf = bsspace[1];
	bsi.bitindex = 0;
	bsi.wordpointer = (unsigned char*) bsbuf;
}

//------------------------------------------------------------------------------

BSBuffer::~BSBuffer()
{

}

//------------------------------------------------------------------------------

void
BSBuffer::FlipInit()
{
	// flip/init buffer for layer 3
	bsbufold = bsbuf;
	bsbuf = bsspace[bsnum] + 512;
	bsnum = (bsnum + 1) & 1;
}
 
//------------------------------------------------------------------------------

unsigned long
BSBuffer::GetBits(int n)
{
	unsigned long rval = bsi.wordpointer[0];
	rval <<= 8;
	rval |= bsi.wordpointer[1];
	rval <<= 8;
	rval |= bsi.wordpointer[2];
	rval <<= bsi.bitindex;
	rval &= 0xffffff;
	bsi.bitindex += n;
	rval >>= (24 - n);
	bsi.wordpointer += (bsi.bitindex >> 3);
	bsi.bitindex &= 7;

	return rval;
}

//------------------------------------------------------------------------------

unsigned long
BSBuffer::GetBitsFast(int n)
{
	unsigned long rval = (unsigned char) (bsi.wordpointer[0] << bsi.bitindex);
	rval |= ((unsigned long) bsi.wordpointer[1] << bsi.bitindex) >> 8;
	rval <<= n;
	rval >>= 8;
	bsi.bitindex += n;
	bsi.wordpointer += (bsi.bitindex >> 3);
	bsi.bitindex &= 7;

	return rval;
}

//------------------------------------------------------------------------------

unsigned char
BSBuffer::Get1Bit()
{
	unsigned char rval_uc = *bsi.wordpointer << bsi.bitindex;
	bsi.bitindex++;
	bsi.wordpointer += (bsi.bitindex>>3);
	bsi.bitindex &= 7;
	rval_uc >>= 7;

	return rval_uc;
}

//------------------------------------------------------------------------------

unsigned int
BSBuffer::GetBitOffset()
{
	return ((-bsi.bitindex) & 0x7);
}

//------------------------------------------------------------------------------

unsigned long
BSBuffer::GetByte()
{
	return *bsi.wordpointer++;
}

//------------------------------------------------------------------------------

void
BSBuffer::BackBits(int n)
{
	bsi.bitindex -= n;
	bsi.wordpointer += (bsi.bitindex >> 3);
	bsi.bitindex &= 0x7;
}

//------------------------------------------------------------------------------

void
BSBuffer::SetPointer(long backstep)
{
	bsi.wordpointer = bsbuf + m_nSSize - backstep;
	if (backstep)
		memcpy(bsi.wordpointer, bsbufold + m_nFSizeOld - backstep, backstep);
	bsi.bitindex = 0;
}

//------------------------------------------------------------------------------

void
BSBuffer::SpaceReset()
{
	memset(bsspace[0], 0, Frame::MAXFRAMESIZE + 512);
	memset(bsspace[1], 0, Frame::MAXFRAMESIZE + 512);
}

//------------------------------------------------------------------------------

void
BSBuffer::Reset()
{
	bsi.bitindex = 0;
	bsi.wordpointer = bsbuf;
}

//------------------------------------------------------------------------------

void
BSBuffer::IncrWordPointer(int n)
{
	bsi.wordpointer += 2;
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
