// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ID3.h
//
// $Id: ID3.h,v 1.2 2002/02/02 18:15:14 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __ID3__
#define __ID3__

//------------------------------------------------------------------------------

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

struct id3_taghdr_t {
	guint8	th_version;
	guint8	th_revision;
	guint8	th_flags;
	guint32	th_size;
};

typedef struct _id3_t id3_t;
typedef struct _id3_frame_t id3_frame_t;
typedef struct _id3_framedesc_t id3_framedesc_t;

//------------------------------------------------------------------------------
/*
 * Structure describing the ID3 tag.
 */
struct _id3_t {
	int		id3_type;		/* Memory or file desriptor */
	int		id3_oflags;		/* Flags from open call */
	int		id3_flags;		/* Flags from tag header */
	int		id3_altered;		/* Set when tag has been altered */
	int		id3_newtag;		/* Set if this is a new tag */
				
	int		id3_version;		/* Major ID3 version number */
	int		id3_revision;		/* ID3 revision number */
				
	int		id3_tagsize;		/* Total size of ID3 tag */
	int		id3_pos;		/* Current position within tag */
    
	char	*id3_error_msg;		/* Last error message */
				
	char	id3_buffer[256];	/* Used for various strings */

	union {
		/*
		 * Memory specific fields.
		 */
		struct {
			void	*id3_ptr;
		} me;
						
		/*
		 * File desc. specific fields.
		 */
		struct {
			int		id3_fd;
			void	*id3_buf;
		} fd;
						
		/*
		 * File ptr. specific fields.
		 */
		struct {
			FILE	*id3_fp;
			void	*id3_buf;
		} fp;
	} s;
				
	/*
	 * Functions for doing operations within ID3 tag.
	 */
	int (*id3_seek)(id3_t *, int);
	void *(*id3_read)(id3_t *, void *, int);
				
	/*
	 * Linked list of ID3 frames.
	 */
	int		id3_numframes;
	id3_frame_t	*id3_frame;
	id3_frame_t	*id3_tail;
};

//------------------------------------------------------------------------------

/*
 * Structure describing an ID3 frame type.
 */
struct _id3_framedesc_t {
	guint32	fd_id;
	char	fd_idstr[4];
	char	*fd_description;
};

//------------------------------------------------------------------------------

/*
 * Structure describing an ID3 frame.
 */

struct _id3_frame_t {
	id3_t		*fr_owner;
	id3_framedesc_t	*fr_desc;
	int			fr_flags;
	unsigned char	fr_encryption;
	unsigned char	fr_grouping;
	unsigned char	fr_altered;

	void		*fr_data;	/* The uncompressed frame */
	int			fr_size;	/* Size of uncompressed frame */
				
	void		*fr_data_z;	/* The compressed frame */
	int			fr_size_z;	/* Size of compressed frame */
				
	id3_frame_t		*fr_next;
};

//------------------------------------------------------------------------------

struct id3tag_t {
	char title[64];
	char artist[64];
	char album[64];
	char comment[256];
	char genre[256];
	gint year;
	gint track_number;
};

//------------------------------------------------------------------------------


#define ID3_SET_SIZE28(size)		\
    ( ((size << 3) & 0x7f000000) |	\
      ((size << 2) & 0x007f0000) |	\
      ((size << 1) & 0x00007f00) |	\
      ((size     ) & 0x0000007f) )

#define ID3_GET_SIZE28(size)		\
    ( ((size & 0x7f000000) >> 3) |	\
      ((size & 0x007f0000) >> 2) |	\
      ((size & 0x00007f00) >> 1) |	\
      ((size & 0x0000007f)     ) )


//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif




