// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Layer3.h
//
// $Id: Layer3.h,v 1.2 2002/01/27 02:12:49 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __Layer3__
#define __Layer3__

#include "Layer.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class Frame;
class Output;
class PlayerInfo;

class Layer3 : public Layer
{
	// Constants
	// Types
	private:
		struct gr_info_s
		{
				int scfsi;
				unsigned part2_3_length;
				unsigned big_values;
				unsigned scalefac_compress;
				unsigned block_type;
				unsigned mixed_block_flag;
				unsigned table_select[3];
				unsigned subblock_gain[3];
				unsigned maxband[3];
				unsigned maxbandl;
				unsigned maxb;
				unsigned region1start;
				unsigned region2start;
				unsigned preflag;
				unsigned scalefac_scale;
				unsigned count1table_select;
				float *full_gain[3];
				float *pow2gain;
		};

		struct III_sideinfo
		{
				unsigned main_data_begin;
				unsigned private_bits;
				struct
				{
						struct gr_info_s gr[2];
				}
				ch[2];
		};
		struct bandInfoStruct
		{
				int longIdx[23];
				int longDiff[22];
				int shortIdx[14];
				int shortDiff[13];
		};

	// Variables
	private:
		float ispow[8207];
		float aa_ca[8], aa_cs[8];
		float COS1[12][6];
		float win[4][36];
		float win1[4][36];
		float gainpow2[256 + 118 + 4];
		float COS9[9];
		float COS6_1, COS6_2;
		float tfcos36[9];
		float tfcos12[3];
#define NEW_DCT9
#ifdef NEW_DCT9
		float cos9[3], cos18[3];
#endif

		float tan1_1[16], tan2_1[16], tan1_2[16], tan2_2[16];
		float pow1_1[2][16], pow2_1[2][16], pow1_2[2][16], pow2_2[2][16];

		int mapbuf0[9][152];
		int mapbuf1[9][156];
		int mapbuf2[9][44];
		int *mapp[9][3];
		int *mapend[9][3];

		int longLimit[9][23];
		int shortLimit[9][14];

		unsigned int n_slen2[512];	/* MPEG 2.0 slen for 'normal' mode */
		unsigned int i_slen2[256];	/* MPEG 2.0 slen for intensity stereo */

		static const bandInfoStruct bandInfo[9];
		
	// Methods
	public:
		Layer3(PlayerInfo* pInfo, Output* pOutput, Visualization* pVis,
			   unsigned char* pSample, int* pPoint, BSBuffer* pBuffer);
		virtual ~Layer3();

		virtual void Init(int nVal);
		
		virtual int Run(Frame* fr);

	private:
		int III_get_side_info(struct III_sideinfo *si, int stereo,
							  int ms_stereo, long sfreq, int single, int lsf);
		int III_get_scale_factors_1(int *scf, struct gr_info_s *gr_info);
		int III_get_scale_factors_2(int *scf, struct gr_info_s *gr_info,
									int i_stereo);
		int III_dequantize_sample(float xr[SBLIMIT][SSLIMIT], int *scf,
								  struct gr_info_s *gr_info, int sfreq,
								  int part2bits);
		void III_i_stereo(float xr_buf[2][SBLIMIT][SSLIMIT], int *scalefac,
						  struct gr_info_s *gr_info, int sfreq, int ms_stereo,
						  int lsf);
		void III_antialias(float xr[SBLIMIT][SSLIMIT], struct gr_info_s *gr_info);
		
		
#ifdef USE_3DNOW
		void dct36(float* inbuf, float* o1, float *o2, float* wintab, float* tsbuf);
#else
		void dct36(float* inbuf, float* o1, float* o2, float* wintab, float* tsbuf);
#endif

		void dct12(float * in, float * rawout1, float * rawout2, register float * wi,
				   register float * ts);
		void III_hybrid(float fsIn[SBLIMIT][SSLIMIT],
						float tsOut[SSLIMIT][SBLIMIT], int ch,
						struct gr_info_s *gr_info, Frame *fr);
		
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
