// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// PlayerOgg.h
//
// $Id: PlayerOgg.h,v 1.4 2002/02/18 09:21:57 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __PlayerOgg__
#define __PlayerOgg__

#include "Player.h"
#include "Thread.h"

#include <cc++/thread.h>
#include <string>
#include <vorbis/vorbisfile.h>

//------------------------------------------------------------------------------

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class PlayerOgg : public Player, Thread
{
	// Constants
	private:
		static const ov_callbacks s_VorbisCallbacks;

	// Types
	// Variables
	private:
		ost::Mutex m_Mutex;
		bool m_bEOS;
		int m_nBytesStreamed;
		bool m_bOutputError;
		std::string m_strFilename;
		int m_nTrack;
		OggVorbis_File m_VF;
		int m_nSampleRate;
		int m_nChannels;
		int m_nSeekNeeded;
		int m_nLength;
		
	// Methods
	public:
		PlayerOgg();
		~PlayerOgg();

		virtual void Init(Output* pOutput, Visualization* pVis);
		
		virtual void Play(const xb::Filename& strFilename, int nTrack);
		virtual void Stop(bool bSendSignal = true);
		virtual void Pause(bool bPause);
		virtual void Seek(int nTime);
		virtual void SeekPercentage(float fPercent);
		virtual int GetTime() const;
		
		virtual bool CanPlayFile(const xb::Filename& strFilename);

		virtual bool CheckCapability(std::string& strMessage);

		virtual void Run();


		static size_t ovcb_read(void *ptr, size_t size, size_t nmemb,
								void *datasource);
		static int ovcb_seek(void *datasource, int64_t offset, int whence);
		static int ovcb_close(void *datasource);
		static long ovcb_tell(void *datasource);
		
	private:
		int ProcessData(int nLastSection);
		void DoSeek();
		void JumpToTime(long nTime);
		
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
