// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// OutputLame.cpp
//
// $Id: OutputLame.cpp,v 1.8 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "OutputLame.h"

#include <string>
#include <xb/ErrorManager.h>
#include <xb/Filename.h>
#include <xb/String.h>

#include "PropList.h"
#include "System.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

OutputLame::OutputLame() :
	Output(),
	m_pLameFlags(NULL),
	m_pFile(NULL),
	m_nFormat(FMT_S16_NE),
	m_nWritten(0)
{

}

//------------------------------------------------------------------------------

OutputLame::~OutputLame()
{
	// delete m_pLameFlags?
}

//------------------------------------------------------------------------------

void
OutputLame::Init()
{

}

//------------------------------------------------------------------------------

bool
OutputLame::Open(AFormat nFormat, int nRate, int nChannels, int nTrack)
{
	xb::Filename strBasePath = System::GetPropList()->GetValue("SpoolRoot");
	std::string strFilename = strBasePath + "track" + xb::String(nTrack) +
		GetFileExtension();
	EM::I()->Trace() << "mp3 filename: " << strFilename << endl;

	m_pFile = fopen(strFilename.c_str(), "wb+");

	if (m_pFile == NULL) {
		return false;
	}

	m_pLameFlags = lame_init();
	AssertNotNull(m_pLameFlags);

	lame_set_num_channels(m_pLameFlags, nChannels);
	
	// 0 = best, 9 = worst
	int nQuality = 100 - System::GetPropList()->GetIntValue("EncoderQuality");
	nQuality /= 10;
	if (nQuality < 0) { nQuality = 0; }
	if (nQuality > 9) { nQuality = 9; }
	EM::I()->Trace() << "Lame encoder quality: " << nQuality << endl;
	lame_set_quality(m_pLameFlags, nQuality);

	if (nChannels == 1) {
		// mono
		lame_set_mode(m_pLameFlags, MONO);
	}
	else if (nChannels == 2) {
		// stereo
		lame_set_mode(m_pLameFlags, JOINT_STEREO);
	}

	int nBitRate = System::GetPropList()->GetIntValue("EncoderBitRate");
	EM::I()->Trace() << "Lame encoder bitrate: " << nBitRate << endl;
	lame_set_brate(m_pLameFlags, nBitRate);
	
	int nResult = lame_init_params(m_pLameFlags);
	if (nResult < 0) {
		// error
	}

	lame_print_config(m_pLameFlags);
	lame_print_internals(m_pLameFlags);

	fprintf(stderr, "sample rate: %d\n", lame_get_out_samplerate(m_pLameFlags));
	
	m_nWritten = 0;
	
	return true;
}

//------------------------------------------------------------------------------

void
OutputLame::Write(void* pData, int nLength)
{

	AssertNotNull(m_pFile);

#ifdef WORDS_BIGENDIAN
	if (m_nFormat == FMT_S8 || m_nFormat == FMT_S16_BE ||
		m_nFormat == FMT_S16_NE || m_nFormat == FMT_U16_LE ||
		m_nFormat == FMT_U16_BE || m_nFormat == FMT_U16_NE) {
#else
	if (m_nFormat == FMT_S8 || m_nFormat == FMT_S16_BE ||
		m_nFormat == FMT_U16_LE || m_nFormat == FMT_U16_BE ||
		m_nFormat == FMT_U16_NE) {
#endif
		ConvertBuffer(pData, nLength);
	}

	gint16* pCData = (gint16 *)pData;
	
    unsigned char pBuffer[LAME_MAXMP3BUFFER];

	int nSamples = nLength / 2;

	// i'm assuming this is nSamples / 2 since shorts are 2 bytes?
	//
	int nSize = lame_encode_buffer_interleaved(
		m_pLameFlags, pCData, nSamples / 2, pBuffer, sizeof(pBuffer));
		
	if (nSize < 0) {
		EM::I()->Error() << "lame_encode_buffer, nSize < 0" << endl;
	}

	m_nWritten += fwrite(pBuffer, 1, nSize, m_pFile);
}

//------------------------------------------------------------------------------

void
OutputLame::Close()
{
	if (m_pFile) {

		// flush
		// 
		unsigned char pBuffer[LAME_MAXMP3BUFFER];
		int nSize = lame_encode_flush(m_pLameFlags, pBuffer, sizeof(pBuffer));

		if (nSize < 0) {
			// errror
		}

		fwrite(pBuffer, 1, nSize, m_pFile);
		
		fclose(m_pFile);

		m_nWritten = 0;
	}

	m_pFile = NULL;
}

//------------------------------------------------------------------------------

int
OutputLame::ElapsedTime()
{
	int nResult = 0;

	/*
	if (m_Header.byte_p_sec != 0.0) {
		nResult = (m_nWritten * 1000) / m_Header.byte_p_sec;
	}
	*/
	
	return nResult;
}

//------------------------------------------------------------------------------

void
OutputLame::ConvertBuffer(gpointer buffer, gint length)
{
	gint i;

	if (m_nFormat == FMT_S8)
	{
		guint8 *ptr1 = (guint8 *)buffer;
		gint8 *ptr2 = (gint8 *)buffer;

		for (i = 0; i < length; i++)
			*(ptr1++) = *(ptr2++) ^ 128;
	}
	if (m_nFormat == FMT_S16_BE)
	{
		gint16 *ptr = (gint16 *)buffer;

		for (i = 0; i < length >> 1; i++, ptr++)
			*ptr = GUINT16_SWAP_LE_BE(*ptr);
	}
	if (m_nFormat == FMT_S16_NE)
	{
		gint16 *ptr = (gint16 *)buffer;

		for (i = 0; i < length >> 1; i++, ptr++)
			*ptr = GINT16_TO_LE(*ptr);
	}
	if (m_nFormat == FMT_U16_BE)
	{
		gint16 *ptr1 = (gint16 *)buffer;
		guint16 *ptr2 = (guint16 *)buffer;

		for (i = 0; i < length >> 1; i++, ptr2++)
			*(ptr1++) = GINT16_TO_LE(GUINT16_FROM_BE(*ptr2) ^ 32768);
	}
	if (m_nFormat == FMT_U16_LE)
	{
		gint16 *ptr1 = (gint16 *)buffer;
		guint16 *ptr2 = (guint16 *)buffer;

		for (i = 0; i < length >> 1; i++, ptr2++)
			*(ptr1++) = GINT16_TO_LE(GUINT16_FROM_LE(*ptr2) ^ 32768);
	}
	if (m_nFormat == FMT_U16_NE)
	{
		gint16 *ptr1 = (gint16 *)buffer;
		guint16 *ptr2 = (guint16 *)buffer;

		for (i = 0; i < length >> 1; i++, ptr2++)
			*(ptr1++) = GINT16_TO_LE((*ptr2) ^ 32768);
	}

}

//------------------------------------------------------------------------------

extern "C" {
	Output* maker()
	{
		return new OutputLame;
	}
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
