// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// OutputOSS.h
//
// $Id: OutputOSS.h,v 1.8 2002/02/17 01:03:50 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __OutputOSS__
#define __OutputOSS__

#include "Output.h"
#include "Thread.h"

#include <glib.h>
#include <string>

#include "Mixer.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class OutputOSS : public Output, Thread
{
	// Constants
	private:
		static const int s_nFrags;
		static const int s_nBufferSize;
		static const int s_nPreBufferSize;
		
	// Types
	// Variables
	private:
		std::string m_strDevice;
		int m_nFile;
		AFormat m_nFormat;
		int m_nOSSFormat;
		int m_nFrequency;
		int m_nEFrequency;
		int m_nChannels;
		int m_nBPS;
		int m_nEBPS;
		int m_nFragSize;
		int m_nBlockSize;

		gpointer m_pBuffer;
		int m_nDeviceBufferSize;
		int m_nBufferSize;
		int m_nPreBufferSize;
		int m_nDeviceBufferUsed;

		AFormat m_nInputFormat;
		int m_nInputChannels;
		int m_nInputBPS;
		int m_nInputFrequency;

		int m_nFlush;
		bool m_bPreBuffer;
		bool m_bRemovePreBuffer;

		bool m_bGoing;
		bool m_bPaused;
		bool m_bDoPause;
		bool m_bUnPause;

		int m_nWriteIndex;
		int m_nReadIndex;
		int m_nOutputTimeOffset;
		guint64 m_nWritten;
		int m_nOutputBytes;

		bool m_bSelectWorks;

		bool m_bSuspend;
		
		Mixer m_Mixer;
		
	// Methods
	public:
		OutputOSS();
		~OutputOSS();

		virtual void Init();

		virtual void GetVolume(int& nLeft, int& nRight);
		virtual void SetVolume(int nLeft, int nRight);

		virtual bool Open(AFormat nFormat, int nRate, int nChannels,
						  int nTrack);
		virtual void Write(void* pData, int nLength);
		virtual void Close();
		virtual void Flush(int nTime);
		virtual void Pause(bool bPause);

		virtual bool BufferPlaying();
		virtual int ElapsedTime();
		virtual guint64 WrittenTime() const;
		virtual int BufferFreeAmt();

		virtual bool IsFileOutput() const { return false; }
		virtual std::string GetFileExtension() const { return ""; }

		virtual std::string GetLibName() const { return "oss"; }
		virtual std::string GetDescription() const { return "OSS audio output"; }
		
		virtual void Run();
		
	private:
		void SetupFormat(AFormat nFormat, int nRate, int nChannels);
		void SetAudioParams();
		int WriteAudio(gpointer pData, int nLength);
		int DownSample(gpointer ob, guint length, guint speed,
					   guint espeed);		
		static void swap_words(guint16 *buffer, gint length);
		int Used() const;
		void CalcDeviceBufferUsed();
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
