// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// OutputWAV.cpp
//
// $Id: OutputWAV.cpp,v 1.4 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "OutputWAV.h"

#include <string>

#include <xb/ErrorManager.h>
#include <xb/Filename.h>
#include <xb/String.h>

#include "PropList.h"
#include "System.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

OutputWAV::OutputWAV()
{

}

//------------------------------------------------------------------------------

OutputWAV::~OutputWAV()
{

}

//------------------------------------------------------------------------------

void
OutputWAV::Init()
{

}

//------------------------------------------------------------------------------

bool
OutputWAV::Open(AFormat nFormat, int nRate, int nChannels, int nTrack)
{
	xb::Filename strBasePath = System::GetPropList()->GetValue("SpoolRoot");
	std::string strWAVFilename = strBasePath + "track" + xb::String(nTrack) +
		".wav";
	EM::I()->Trace() << "wav filename: " << strWAVFilename << endl;

	m_pFile = fopen(strWAVFilename.c_str(), "wb");

	if (m_pFile == NULL) {
		return false;
	}

	m_nFormat = nFormat;
	m_nWritten = 0;
	
	memcpy(&m_Header.main_chunk, "RIFF", 4);
	m_Header.length = GUINT32_TO_LE(0);
	memcpy(&m_Header.chunk_type, "WAVE", 4);
	memcpy(&m_Header.sub_chunk, "fmt ", 4);
	m_Header.sc_len = GUINT32_TO_LE(16);
	m_Header.format = GUINT16_TO_LE(1);
	m_Header.modus = GUINT16_TO_LE(nChannels);
	m_Header.sample_fq = GUINT32_TO_LE(nRate);
	if (nFormat == FMT_U8 || nFormat == FMT_S8)
		m_Header.bit_p_spl = GUINT16_TO_LE(8);
	else
		m_Header.bit_p_spl = GUINT16_TO_LE(16);
	m_Header.byte_p_sec = GUINT32_TO_LE(nRate * m_Header.modus *
										(GUINT16_FROM_LE(m_Header.bit_p_spl) / 8));
	m_Header.byte_p_spl = GUINT16_TO_LE((GUINT16_FROM_LE(m_Header.bit_p_spl) /
										 (8 / nChannels)));
	memcpy(&m_Header.data_chunk, "data", 4);
	m_Header.data_length = GUINT32_TO_LE(0);
	fwrite(&m_Header, sizeof (struct wavhead), 1, m_pFile);

	return true;
}

//------------------------------------------------------------------------------

void
OutputWAV::Write(void* pData, int nLength)
{

	AssertNotNull(m_pFile);
	
#ifdef WORDS_BIGENDIAN
	if (m_nFormat == FMT_S8 || m_nFormat == FMT_S16_BE ||
		m_nFormat == FMT_S16_NE || m_nFormat == FMT_U16_LE ||
		m_nFormat == FMT_U16_BE || m_nFormat == FMT_U16_NE) {
#else
	if (m_nFormat == FMT_S8 || m_nFormat == FMT_S16_BE ||
		m_nFormat == FMT_U16_LE || m_nFormat == FMT_U16_BE ||
		m_nFormat == FMT_U16_NE) {
#endif
		ConvertBuffer(pData, nLength);
	}

	m_nWritten += fwrite(pData, 1, nLength, m_pFile);

}

//------------------------------------------------------------------------------

void
OutputWAV::Close()
{
	if (m_pFile) {
		m_Header.length = GUINT32_TO_LE(m_nWritten +
										sizeof (struct wavhead) - 8);

		m_Header.data_length = GUINT32_TO_LE(m_nWritten);
		fseek(m_pFile, 0, SEEK_SET);
		fwrite(&m_Header, sizeof (struct wavhead), 1, m_pFile);

		fclose(m_pFile);

		EM::I()->Trace() << "m_Header.length:  " <<
			m_nWritten + sizeof(struct wavhead) - 8 << endl;
		EM::I()->Trace() << "m_Header.data_length:  " << m_nWritten << endl;
		
		
		m_nWritten = 0;
	}

	m_pFile = NULL;
}

//------------------------------------------------------------------------------

int
OutputWAV::ElapsedTime()
{
	int nResult = 0;

	if (m_Header.byte_p_sec != 0.0) {
		nResult = (m_nWritten * 1000) / m_Header.byte_p_sec;
	}
	
	return nResult;
}

//------------------------------------------------------------------------------

void
OutputWAV::ConvertBuffer(gpointer buffer, gint length)
{
	gint i;

	if (m_nFormat == FMT_S8)
	{
		guint8 *ptr1 = (guint8 *)buffer;
		gint8 *ptr2 = (gint8 *)buffer;

		for (i = 0; i < length; i++)
			*(ptr1++) = *(ptr2++) ^ 128;
	}
	if (m_nFormat == FMT_S16_BE)
	{
		gint16 *ptr = (gint16 *)buffer;

		for (i = 0; i < length >> 1; i++, ptr++)
			*ptr = GUINT16_SWAP_LE_BE(*ptr);
	}
	if (m_nFormat == FMT_S16_NE)
	{
		gint16 *ptr = (gint16 *)buffer;

		for (i = 0; i < length >> 1; i++, ptr++)
			*ptr = GINT16_TO_LE(*ptr);
	}
	if (m_nFormat == FMT_U16_BE)
	{
		gint16 *ptr1 = (gint16 *)buffer;
		guint16 *ptr2 = (guint16 *)buffer;

		for (i = 0; i < length >> 1; i++, ptr2++)
			*(ptr1++) = GINT16_TO_LE(GUINT16_FROM_BE(*ptr2) ^ 32768);
	}
	if (m_nFormat == FMT_U16_LE)
	{
		gint16 *ptr1 = (gint16 *)buffer;
		guint16 *ptr2 = (guint16 *)buffer;

		for (i = 0; i < length >> 1; i++, ptr2++)
			*(ptr1++) = GINT16_TO_LE(GUINT16_FROM_LE(*ptr2) ^ 32768);
	}
	if (m_nFormat == FMT_U16_NE)
	{
		gint16 *ptr1 = (gint16 *)buffer;
		guint16 *ptr2 = (guint16 *)buffer;

		for (i = 0; i < length >> 1; i++, ptr2++)
			*(ptr1++) = GINT16_TO_LE((*ptr2) ^ 32768);
	}

}

//------------------------------------------------------------------------------

extern "C" {
	Output* maker()
	{
		return new OutputWAV;
	}
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
