// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// OutputWAV.h
//
// $Id: OutputWAV.h,v 1.5 2002/02/17 01:03:50 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __OutputWAV__
#define __OutputWAV__

#include <glib.h>
#include <stdio.h>

#include "Output.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class OutputWAV : public Output
{
	// Constants
	// Types
	private:
		
		struct wavhead
		{
				guint32 main_chunk;
				guint32 length;
				guint32 chunk_type;
				guint32 sub_chunk;
				guint32 sc_len;
				guint16 format;
				guint16 modus;
				guint32 sample_fq;
				guint32 byte_p_sec;
				guint16 byte_p_spl;
				guint16 bit_p_spl;
				guint32 data_chunk;
				guint32 data_length;
		};

	// Variables
	private:
		wavhead m_Header;
		FILE* m_pFile;
		AFormat m_nFormat;
		int m_nWritten;
		
	// Methods
	public:
		OutputWAV();
		~OutputWAV();

		virtual void Init();

		virtual void GetVolume(int& nLeft, int& nRight) {}
		virtual void SetVolume(int nLeft, int nRight) {}

		virtual bool Open(AFormat nFormat, int nRate, int nChannels,
						  int nTrack);
		virtual void Write(void* pData, int nLength);
		virtual void Close();
		virtual void Flush(int nTime) {}
		virtual void Pause(bool bPause) {}

		virtual bool BufferPlaying() { return false; }
		virtual int ElapsedTime();
		virtual guint64 WrittenTime() const { return 0; }
		virtual int BufferFreeAmt() { return 1000000; }

		virtual bool IsFileOutput() const { return true; }
		virtual std::string GetFileExtension() const { return ".wav"; }

		virtual std::string GetLibName() const { return "wav"; }
		virtual std::string GetDescription() const { return "WAV file output"; }
		
	private:
		void ConvertBuffer(gpointer buffer, gint length);
		
		
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
