// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ActiveList.h
//
// $Id: ActiveList.h,v 1.19 2002/02/18 21:00:30 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __ActiveList__
#define __ActiveList__

#include <xb/Pixmap.h>
#include <xb/XCList.h>

#include "Song.h"
#include "SongList.h"

//------------------------------------------------------------------------------

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class ActiveList : public xb::XCList
{
	// Constants
	// Types
	// Variables
	private:
		xb::Pixmap m_PMCboxNoAudioEmpty;
		xb::Pixmap m_PMCboxAudioEmpty;
		xb::Pixmap m_PMCboxNoAudioFull;
		xb::Pixmap m_PMCboxAudioFull;

		Gdk_Color m_ColorOdd;
		Gdk_Color m_ColorEven;

		bool m_bReorderable;

		unsigned int m_nSortColumn;
		GtkSortType m_nSortType;

		Song m_SongPlaying;
		bool m_bSongPlaying;
		
	// Methods
	public:
		ActiveList();
		~ActiveList();

		void AppendSong(const Song& song);
		void SetCheckbox(unsigned int nRow, bool bOn);
		void SetAudio(unsigned int nRow, bool bOn);

		void DNDDataGet(GdkDragContext* pContext,
						GtkSelectionData* pSelectionData,
						guint info,
						guint time);

		virtual void clear();
		void GetSong(int nRow, Song& song) const;
		bool IsSelected(int nRow) const;

		void GetAllSongs(SongList& listSongs) const;
		void GetSelectedSongs(SongList& listSongs) const;
		int GetSelectionListIndex(unsigned int nNum) const;

		bool SetSongPlaying(const Song& song);
		void SetNoSongPlaying();
		bool UpdateSongPlaying();

		void CheckAll();
		
		// void SetSearch(const std::string strSearch);
		
		void SetReorderable(bool bIn);
		
		SigC::Signal0<void> Reordered;


	private:
		void DNDReceive(GdkDragContext* pContext, gint x, gint y,
						GtkSelectionData* pData,
						guint info,
						guint32 time);
		void DNDDelete(GdkDragContext* pContext);
		Song* GetSongPtr(unsigned int nRow);
		Song* GetSongPtr(unsigned int nRow) const;

		void ColumnClick(int nColumn);
		void UpdateBackgroundColors();

		void ClearData();
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
