// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Library.h
//
// $Id: Library.h,v 1.16 2002/02/18 08:55:59 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __LIBRARY__
#define __LIBRARY__

//------------------------------------------------------------------------------

#include <sigc++/signal_system.h>
#include <sigc++/object.h>
#include <string>
#include <vector>
#include <xb/Filename.h>

#include "Source.h"

#include "Song.h"
#include "SongList.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class Library : public Source, public SigC::Object
{
	// Constants
		
	// Types
		
	// Variables
	private:
		xb::Filename m_strRoot;
		bool m_bDirty;
		xb::Filename m_strCacheFilename;
		unsigned int m_nIDCount;
		
	// Methods
	public:
		Library(const std::string& strRoot);
		~Library();

		void Dump() const;

		xb::Filename GetRoot() const { return m_strRoot; }
		
		xb::Filename GetFilename(const Song& song) const;

		virtual const SongList& GetSongList();
		std::string Import(const Song& song, const std::string& strInFilename,
						   bool bLink);
		std::string Import(const std::string& strFilename, bool bLink);
		void ImportDirectory(const std::string& strFilename, bool bLink);

		void Remove(const Song& song);

		void SetCacheUpdate(bool bIn);

		void ResetTimes();
		void ClearCache();
		
	private:
		void Update();

		void AddSong(const std::string& strFilename,
					 const std::string& strTitleIn,
					 const std::string& strArtist,
					 const std::string& strAlbum);
		
		void WriteCache() const;
		bool ReadCache();

		bool CacheUpdateNeeded() const;
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------

