// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// MP3Info.h
//
// $Id: MP3Info.h,v 1.7 2002/02/18 08:55:59 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __MP3Info__
#define __MP3Info__

#include <stdio.h>
#include <string>
#include <xb/Filename.h>

#include "Song.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class MP3Info
{
	// Constants
	private:
		static const int NUM_SAMPLES = 4;
		static const int FRAME_HEADER_SIZE = 4;
		static const int MIN_FRAME_SIZE = 21;
		static const int MIN_CONSEC_GOOD_FRAMES = 21;

		static const int frame_size_index[3];
		static const int bitrate[2][3][14];
		static const int frequencies[3][4];
		
	// Types
	private:
		struct MP3Header {
				unsigned long m_nSync;
				unsigned int m_nVersion;
				unsigned int m_nLayer;
				unsigned int m_nCRC;
				unsigned int m_nBitRate;
				unsigned int m_nFreq;
				unsigned int m_nPadding;
				unsigned int m_nExtension;
				unsigned int m_nMode;
				unsigned int m_nModeExtension;
				unsigned int m_nCopyright;
				unsigned int m_nOriginal;
				unsigned int m_nEmphasis;
		};
		
	// Variables
	private:
		// header
		MP3Header m_Header;
		
		// id3 tag
		char m_strCTitle[31];
		std::string m_strTitle;
		char m_strCArtist[31];
		std::string m_strArtist;
		char m_strCAlbum[31];
		std::string m_strAlbum;
		char m_strYear[5];
		char m_strComment[31];
		unsigned char m_track[1];
		unsigned char m_genre[1];

		// info
		xb::Filename m_strFilename;
		FILE* m_pFile;
		off_t m_DataSize;
		int m_nHeaderIsValid;
		int m_nID3IsValid;
		int m_nVBR;
		float m_fVBRAverage;
		int m_nSeconds;
		int m_nFrames;
		int m_nBadFrames;
		
	// Methods
	public:
		MP3Info(const xb::Filename& strFilename);
		~MP3Info();

		void Dump() const;

		int GetSeconds() const { return m_nSeconds; }
		std::string GetTitle() const { return m_strTitle; }
		std::string GetArtist() const { return m_strArtist; }
		std::string GetAlbum() const { return m_strAlbum; }
		std::string GetTrack() const { return std::string(""); }

		void WriteTag(const Song& song);
	
	private:
		bool LoadMP3Info(bool bFullScanVBR = true);
		static char* unpad(char* str);
		static char* pad(char* str, int length);

		int HeaderBitRate(MP3Header* pH);
		int FrameLength(MP3Header* pH);
		bool GetFirstHeader(long nStartPos);
		bool LoadID3();
		int GetHeader(MP3Header* pH);
		bool SameConstant(MP3Header* pH1, MP3Header* pH2);
		int HeaderBitrate(MP3Header* pH);
		int HeaderFrequency(MP3Header* pH);
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
