// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// MWStateEQ.cpp
//
// $Id: MWStateEQ.cpp,v 1.7 2002/02/18 08:55:59 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "MWStateEQ.h"

#include <gtk--/style.h>
#include <math.h>
#include <xb/ErrorManager.h>

#include "Controller.h"
#include "fft.h"
#include "System.h"
#include "WidgetMsgWin.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

const int MWStateEQ::s_nBarWidth = 7;
const int MWStateEQ::s_nBarPad = 3;

const int MWStateEQ::s_anXScale[21] = {
	0, 1, 2, 3, 4, 5, 6, 7, 8, 11, 15, 20, 27, 36, 47, 62, 82, 107, 141, 184,
	255
};
const int MWStateEQ::s_nXScaleMax;
const double MWStateEQ::s_fYScale = 3.60673760222;

//------------------------------------------------------------------------------

//------------------------------------------------------------------------------

MWStateEQ::MWStateEQ(WidgetMsgWin* pParent) :
	MWState(pParent)
{
	m_pTimer = g_timer_new();
	g_timer_start(m_pTimer);
	
	for (int i = 0; i < s_nXScaleMax; i++) {
		m_aCaps[i].fPercent = 0.0;
		m_aCaps[i].fTime = g_timer_elapsed(m_pTimer, NULL);
	}
}

//------------------------------------------------------------------------------

MWStateEQ::~MWStateEQ()
{
	g_timer_destroy(m_pTimer);
	m_pTimer = NULL;
}

//------------------------------------------------------------------------------

void
MWStateEQ::Draw(Gdk_Pixmap* pPixmap)
{
	const gint16* pData = System::GetController()->GetVisualizationData();
	int nChannels = System::GetController()->GetVisualizationChannels();

	
	if (pData == NULL) {
		return;
	}

	int nPadX = GetPadX();
	
	gint16* anMonoFreq = new gint16[2 * 256];
	
	CalcMonoFreq(anMonoFreq, pData, nChannels);

	gchar intern_vis_data[512];

	for (int i = 0; i < s_nXScaleMax; i++) {
		
		int nY = 0;
		for (int j = s_anXScale[i]; j < s_anXScale[i + 1]; j++) {
			if (anMonoFreq[j] > nY) { // 0 * 256 + j
				nY = anMonoFreq[j]; // 0 * 256 + j
			}
		}

		nY >>= 7;
		if (nY != 0) {
			intern_vis_data[i] = (char)(log(nY) * s_fYScale);
			if (intern_vis_data[i] > 15) {
				intern_vis_data[i] = 15;
			}
		}
		else {
			intern_vis_data[i] = 0;
		}
		
		float fPercent = (float)(intern_vis_data[i] / 15.0);
		DrawBar(pPixmap, nPadX + i * (s_nBarWidth + s_nBarPad), fPercent);

		if (fPercent > m_aCaps[i].fPercent) {
			SetCap(i, fPercent);
		}
	}
	
	delete anMonoFreq;

	DrawCaps(pPixmap);
	DrawEQButton(pPixmap);
	
}

//------------------------------------------------------------------------------

int
MWStateEQ::GetPadX() const
{
	int nWidth = m_pParent->width();
	int nGraphWidth = (s_nBarWidth + s_nBarPad) * s_nXScaleMax;
	return (nWidth - nGraphWidth) / 2;
}

//------------------------------------------------------------------------------

void
MWStateEQ::GetBarHeightY(float fPercent, int& nY, int& nHeight)
{
	int nMaxHeight = m_pParent->height();
	int nPadY = 10;
	nY = nPadY;
	int nY2 = nMaxHeight - nPadY;
	int nYRange = nY2 - nY;
	nHeight = (int)(nYRange * fPercent);
	if (nHeight < 1) {
		nHeight = 1;
	}
	nY = nY2 - nHeight;
}

//------------------------------------------------------------------------------

void
MWStateEQ::DrawBar(Gdk_Pixmap* pPixmap, int nX, float fPercent)
{
	Gdk_GC gc = m_pParent->get_style()->get_black_gc();

	int nY1, nHeight;
	GetBarHeightY(fPercent, nY1, nHeight);
	
	pPixmap->draw_rectangle(gc, true, nX, nY1, s_nBarWidth, nHeight);
}

//------------------------------------------------------------------------------

void
MWStateEQ::DrawCaps(Gdk_Pixmap* pPixmap)
{
	Gdk_GC gc = m_pParent->get_style()->get_black_gc();

	double dTime = g_timer_elapsed(m_pTimer, NULL);
	
	for (int i = 0; i < s_nXScaleMax; i++) {
		int nX = GetPadX() + i * (s_nBarWidth + s_nBarPad);

		int nY, nHeight;
		GetBarHeightY(m_aCaps[i].fPercent, nY, nHeight);

		pPixmap->draw_rectangle(gc, true, nX, nY, s_nBarWidth, 1);

		/*
		m_aCaps[i].nTime--;
		if (m_aCaps[i].nTime <= 0) {
			m_aCaps[i].nTime = 0;
			if (m_aCaps[i].fPercent > 0.0) {
				m_aCaps[i].fPercent -= 0.05;
				if (m_aCaps[i].fPercent < 0.0) {
					m_aCaps[i].fPercent = 0.0;
				}
			}
		}
		*/

		double dTimeElapsed = dTime - m_aCaps[i].fTime;
		
		if (dTimeElapsed > 0.3) {
			// m_aCaps[i].fTime = g_timer_elapsed(m_pTimer, NULL);
			if (m_aCaps[i].fPercent > 0.0) {
				m_aCaps[i].fPercent -= 0.3 * dTimeElapsed;
				if (m_aCaps[i].fPercent < 0.0) {
					m_aCaps[i].fPercent = 0.0;
				}
			}
		}
	}
}

//------------------------------------------------------------------------------

void
MWStateEQ::SetCap(int nX, float fPercent)
{
	m_aCaps[nX].fPercent = fPercent;
	m_aCaps[nX].fTime = g_timer_elapsed(m_pTimer, NULL);
}

//------------------------------------------------------------------------------

void
MWStateEQ::CalcFreq(gint16 *dest, const gint16 *src)
{
	static fft_state *state = NULL;
	gfloat tmp_out[257];
	gint i;
	
	if(!state)
		state = fft_init();

	fft_perform(src,tmp_out,state);
	
	for(i = 0; i < 256; i++)
		dest[i] = ((gint)sqrt(tmp_out[i + 1])) >> 8;
}

//------------------------------------------------------------------------------

void
MWStateEQ::CalcMonoFreq(gint16* dest, const gint16* src, gint nch)
{
	gint i;
	gint16 *d;
	const gint16* sl;
	const gint16* sr;
	gint16 tmp[512];

	if(nch == 1)
		CalcFreq((gint16*)(dest[0]), (gint16*)(src[0]));
	else
	{
		d = tmp;
		sl = src;
		sr = src + 512;
		for(i = 0; i < 512; i++)
		{
			*d = (*sl + *sr) >> 1;
			d++;
			sl++;
			sr++;
		}
		CalcFreq(dest, (gint16*)tmp);
	}
}

//------------------------------------------------------------------------------

void
MWStateEQ::ButtonPress(int nX, int nY)
{
	if (nX >= m_nEQButtonX &&
		nX < m_nEQButtonX + m_pixEQButton.GetWidth() &&
		nY >= m_nEQButtonY &&
		nY < m_nEQButtonY + m_pixEQButton.GetHeight()) {
		EQButtonClicked();
	}
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
