// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// MWStatePlay.cpp
//
// $Id: MWStatePlay.cpp,v 1.7 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include <config.h>

#include "MWStatePlay.h"

#include <gtk--/style.h>
#include <xb/ErrorManager.h>

#include "Util.h"
#include "WidgetMsgWin.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

const int MWStatePlay::s_nBackColorRed = 35723;
const int MWStatePlay::s_nBackColorGreen = 48573;
const int MWStatePlay::s_nBackColorBlue = 34952;

//------------------------------------------------------------------------------

MWStatePlay::MWStatePlay(WidgetMsgWin* pParent) :
	MWState(pParent),
	m_nMS(0)
{
	m_pTimer = g_timer_new();
	g_timer_start(m_pTimer);
}

//------------------------------------------------------------------------------

MWStatePlay::~MWStatePlay()
{
	g_timer_destroy(m_pTimer);
	m_pTimer = NULL;
}

//------------------------------------------------------------------------------

void
MWStatePlay::Draw(Gdk_Pixmap* pPixmap)
{
	DrawTitle(pPixmap);

	int nElapsedTime = m_nMS / 1000;
	std::string s("Elapsed time:  ");
	s += Util::GetTimeStr(nElapsedTime);
	float fPercentage = (float)nElapsedTime / m_Song.GetSeconds();
	if (fPercentage < 0.0) { fPercentage = 0.0; }
	if (fPercentage > 1.0) { fPercentage = 1.0; }

	DrawProgressBar(pPixmap, fPercentage);
	DrawTime(pPixmap, s);
	DrawEQButton(pPixmap);
}

//------------------------------------------------------------------------------


void
MWStatePlay::SetSong(const Song& song)
{
	m_Song = song;

	ResetFade();
	ResetScroll();
}

//------------------------------------------------------------------------------

void
MWStatePlay::DrawTitle(Gdk_Pixmap* pPixmap)
{
	if (m_nTextState == NORMAL) {
		DrawCenteredScrollString(pPixmap,
								 m_pParent->get_style()->get_black_gc(),
								 m_strTitleText, 20);
	}
	else if (m_nTextState == FADE_OUT) {
		m_nTextRed = FadeColor(m_nTextRed, s_nBackColorRed);
		m_nTextGreen = FadeColor(m_nTextGreen, s_nBackColorGreen);
		m_nTextBlue = FadeColor(m_nTextBlue, s_nBackColorBlue);

		if (m_nTextRed > s_nBackColorRed) {
			m_nTextRed = s_nBackColorRed;
		}
		if (m_nTextGreen > s_nBackColorGreen) {
			m_nTextGreen = s_nBackColorGreen;
		}
		if (m_nTextBlue > s_nBackColorBlue) {
			m_nTextBlue = s_nBackColorBlue;
		}
		
		Gdk_GC gc;
		gc.create(*pPixmap);

		SetColor(gc, m_nTextRed, m_nTextGreen, m_nTextBlue);
		DrawCenteredScrollString(pPixmap, gc, m_strTitleText, 20);

		if (m_nTextRed >= s_nBackColorRed) {
			m_nTextState = FADE_IN;
			m_nTitleType = (TitleType)((m_nTitleType + 1) % NUM_TITLETYPES);
			if (m_nTitleType == SONGTITLE) {
				m_strTitleText = m_Song.GetTitle();
			}
			else if (m_nTitleType == ARTIST) {
				m_strTitleText = m_Song.GetArtist();
			}
			else if (m_nTitleType == ALBUM) {
				m_strTitleText = m_Song.GetAlbum();
			}
			else {
				EM::I()->Warn() << "unhandled title type: " << m_nTitleType <<
					endl;
			}
		}
		
	}
	else if (m_nTextState == FADE_IN) {
		m_nTextRed = FadeColor(m_nTextRed, 0);
		m_nTextGreen = FadeColor(m_nTextGreen, 0);
		m_nTextBlue = FadeColor(m_nTextBlue, 0);

		if (m_nTextRed < 0) { m_nTextRed = 0; }
		if (m_nTextGreen < 0) { m_nTextGreen = 0; }
		if (m_nTextBlue < 0) { m_nTextBlue = 0; }
		
		Gdk_GC gc;
		gc.create(*pPixmap);

		SetColor(gc, m_nTextRed, m_nTextGreen, m_nTextBlue);
		DrawCenteredScrollString(pPixmap, gc, m_strTitleText, 20);

		if (m_nTextRed == 0) {
			m_nTextState = NORMAL;
		}

	}

	if (m_nTextState == NORMAL) {
		/*
		m_nTimeCount++;

		if (m_nTimeCount > 50) {
			m_nTextState = FADE_OUT;
			m_nTimeCount = 0;
		}
		*/

		double dTime = g_timer_elapsed(m_pTimer, NULL);
		if (dTime > 5) {
			m_nTextState = FADE_OUT;
			g_timer_reset(m_pTimer);
		}
	}
}

//------------------------------------------------------------------------------

void
MWStatePlay::DrawTime(Gdk_Pixmap* pPixmap, const std::string& strTime)
{
	DrawCenteredHString(pPixmap, m_pParent->get_style()->get_black_gc(),
						strTime, 60, 10);
}

//------------------------------------------------------------------------------

void
MWStatePlay::ResetFade()
{
	m_strTitleText = m_Song.GetTitle();
	m_nTitleType = SONGTITLE;
	m_nTextRed = 0;
	m_nTextGreen = 0;
	m_nTextBlue = 0;
	m_nTimeCount = 0;
	m_nTextState = NORMAL;
}

//------------------------------------------------------------------------------

int
MWStatePlay::FadeColor(int nSource, int nDest)
{
	int nDistance = nDest - nSource;
	int nStep = (int)(0.4 * nDistance);

	if (nSource < nDest) {
		if (nStep < 10) { nStep = 10; }
	}
	else {
		if (nStep > -10) { nStep = -10; }
	}
	
	return nSource + nStep;
}

//------------------------------------------------------------------------------

void
MWStatePlay::ButtonPress(int nX, int nY)
{
	if (nX >= m_nProgressX && nX < m_nProgressX + m_nProgressWidth &&
		nY >= m_nProgressY && nY < m_nProgressY + m_nProgressHeight) {
		float fPercentage =
			((float)(nX - m_nProgressX)) / ((float)m_nProgressWidth);
		
		ProgressClicked(fPercentage);
	}
	else if (nX >= m_nEQButtonX &&
			 nX < m_nEQButtonX + m_pixEQButton.GetWidth() &&
			 nY >= m_nEQButtonY &&
			 nY < m_nEQButtonY + m_pixEQButton.GetHeight()) {
		EQButtonClicked();
	}
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
