// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// MWStateProgress.cpp
//
// $Id: MWStateProgress.cpp,v 1.1 2002/01/29 02:39:54 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "MWStateProgress.h"

#include <gtk--/style.h>

#include "WidgetMsgWin.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

MWStateProgress::MWStateProgress(WidgetMsgWin* pParent) :
	MWState(pParent),
	m_fPercentage(0.0),
	m_fStep(0.0)
{

}

//------------------------------------------------------------------------------

MWStateProgress::~MWStateProgress()
{

}

//------------------------------------------------------------------------------

void
MWStateProgress::Draw(Gdk_Pixmap* pPixmap)
{
	DrawCenteredHString(pPixmap, m_pParent->get_style()->get_black_gc(),
						m_strProgressText,
						20, 10);

	DrawProgressBar(pPixmap, m_fPercentage);

	DrawCancelButton(pPixmap, 50);
	
	float fMultiplier = 0.4;
	float fMinimum = 0.01;
	float fMaximum = 0.04;
	
	// calculate sliding
	if (m_fStep < 0) {
		// moving left
		if (m_fPercentage <= 0.0) {
			m_fStep = -m_fStep;
		}
		else {
			float fDistance = m_fPercentage;
			if (fDistance > fMinimum) {
				m_fStep = -fMultiplier * fDistance;
				if (m_fStep < -fMaximum) {
					m_fStep = -fMaximum;
				}
			}
			else {
				m_fStep = -fMinimum;
			}
		}
	}
	else {
		// moving right
		if (m_fPercentage >= 1.0) {
			m_fStep = -m_fStep;
		}
		else {
			float fDistance = 1.0 - m_fPercentage;
			if (fDistance > fMinimum) {
				m_fStep = fMultiplier * fDistance;
				if (m_fStep > fMaximum) {
					m_fStep = fMaximum;
				}
			}
			else {
				m_fStep = fMinimum;
			}
		}
	}

	m_fPercentage += m_fStep;
}

//------------------------------------------------------------------------------

void
MWStateProgress::DrawCancelButton(Gdk_Pixmap* pPixmap, int nY)
{
	int nSize = 16;
	int nX = (m_pParent->width() - nSize) / 2;

	pPixmap->draw_rectangle(m_pParent->get_style()->get_black_gc(), false,
							nX, nY, nSize, nSize);
	pPixmap->draw_line(m_pParent->get_style()->get_black_gc(),
					   nX, nY,	nX + nSize, nY + nSize);
	pPixmap->draw_line(m_pParent->get_style()->get_black_gc(),
					   nX, nY + nSize, nX + nSize, nY);

	m_nCancelX = nX;
	m_nCancelY = nY;
	
	/*
	Gdk_GC gc;
	
	gc.create(m_pixBack);
	Gdk_Pixmap pix = m_pixCancel.GetPixmap();
	m_pixBack.draw_pixmap(gc, pix, 0, 0, nX, nY);
	*/
}

//------------------------------------------------------------------------------

void
MWStateProgress::SetProgressText(const std::string& strText)
{
	m_strProgressText = strText;
	m_fPercentage = 0.0;
	m_fStep = 1.0;
}

//------------------------------------------------------------------------------

void
MWStateProgress::ButtonPress(int nX, int nY)
{
	if (nX >= m_nCancelX && nX < m_nCancelX + 16 &&
		nY >= m_nCancelY && nY < m_nCancelY + 16) {
		CancelClicked();
	}
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
