// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// PlayList.cpp
//
// $Id: PlayList.cpp,v 1.16 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "PlayList.h"

#include <fstream>
#include <stdio.h>
#include <xb/ErrorManager.h>
#include <xb/FileUtil.h>

#include "Library.h"
#include "System.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

PlayList::PlayList() :
	m_bLoaded(false)
{

}

//------------------------------------------------------------------------------

PlayList::~PlayList()
{

}

//------------------------------------------------------------------------------

void
PlayList::Load()
{
	AssertNotNull(System::GetLibrary());

	m_SongList.clear();
	
	std::ifstream infile(m_strFilename.c_str());

	std::string s;

	while (getline(infile, s)) {
		if (s.length() > 0) {
			EM::I()->Trace() << "Playlist file: " << s << endl;
			Song song(s);
			Add(song);
		}
	}

	m_bLoaded = true;
}

//------------------------------------------------------------------------------

void
PlayList::Save() const
{
	EM::I()->Trace() << "saving " << m_strFilename << endl;

	if (m_SongList.empty()) {
		EM::I()->Warn() << "play list is empty, not saving." << endl;
		return;
	}
	
	ofstream outfile(m_strFilename.c_str());
	if (!outfile.is_open()) {
		throw(xb::file_error("couldn't open " + m_strFilename));
	}

	SongList::const_iterator itr;
	for (itr = m_SongList.begin(); itr != m_SongList.end(); ++itr) {
		outfile << itr->GetFilename() << endl;
		EM::I()->Trace() << "Playlist file: " << itr->GetFilename() << endl;
	}

	EM::I()->Trace() << "done saving." << endl;
}

//------------------------------------------------------------------------------

void
PlayList::Rename(const std::string& s)
{
	Save();
	if (!xb::FileUtil::FileExists(s)) {
		EM::I()->Trace() << "renaming " << m_strFilename << " to " << s << endl;
		xb::FileUtil::Rename(m_strFilename, s);
	}
	else {
		throw(xb::file_error("File " + s + " doesn't exist."));
	}
}

//------------------------------------------------------------------------------

void
PlayList::Delete()
{
	xb::FileUtil::Delete(m_strFilename);
}

//------------------------------------------------------------------------------

void
PlayList::Add(const Song& song)
{
	m_SongList.push_back(song);
}

//------------------------------------------------------------------------------

void
PlayList::Remove(const Song& song)
{
	bool bFound = false;
	SongList::iterator itr;
	for (itr = m_SongList.begin(); itr != m_SongList.end() && !bFound; ++itr) {
		if (*itr == song) {
			EM::I()->Trace() << "removing " << song.GetTitle() <<
				" from playlist." << endl;
			m_SongList.erase(itr);
			bFound = true;
		}
	}
	
	if (!bFound) {
		EM::I()->Warn() << "couldn't find " << song.GetTitle() <<
			" in playlist." << endl;
	}
}

//------------------------------------------------------------------------------

void
PlayList::Move(int nOrig, int nNew)
{
	/*
	EM::I()->Trace("moving elt " + String(nOrig) + " to " + String(nNew));

	Song s = m_SongList[nOrig];
	*/
}

//------------------------------------------------------------------------------

void
PlayList::SetSongList(const SongList& listSongs)
{
	m_SongList = listSongs;
}

//------------------------------------------------------------------------------

void
PlayList::CheckLoad()
{
	if (!m_bLoaded) {
		EM::I()->Trace() << "playlist " << m_strFilename <<
			" not loaded...loading." << endl;
		Load();
	}
}

//------------------------------------------------------------------------------

const SongList&
PlayList::GetSongList()
{
	CheckLoad();
	return Source::GetSongList();
}

//------------------------------------------------------------------------------

int
PlayList::GetNumSongs()
{
	CheckLoad();
	return Source::GetNumSongs();
}

//------------------------------------------------------------------------------

PlayList&
PlayList::operator=(const PlayList& rhs)
{
	if (&rhs != this) {
		m_strName = rhs.m_strName;
		m_strFilename = rhs.m_strFilename;
		m_bLoaded = rhs.m_bLoaded;
	}

	return *this;
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
