// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Song.h
//
// $Id: Song.h,v 1.13 2002/02/18 08:55:59 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __SONG__
#define __SONG__

#include <fstream>
#include <string>
#include <xb/Filename.h>

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class Song
{
		// Constants
		// Types
		// Variables
	private:
		unsigned int m_nID;
		std::string m_strArtist;
		std::string m_strAlbum;
		std::string m_strTitle;
		std::string m_strTrack;
		std::string m_strTime;
		xb::Filename m_strFilename;
		unsigned int m_nSeconds;
		bool m_bSelected;
		bool m_bPlaying;
		
		// Methods
	public:
		Song();
		Song(const std::string& strArtist,
			 const std::string& strAlbum,
			 const std::string& strTitle,
			 const std::string& strTrack,
			 const std::string& strTime,
			 unsigned int nSeconds,
			 bool bSelected = false,
			 bool bPlaying = false);
		Song(const xb::Filename& strFilename);
		~Song();

		inline unsigned int GetID() const { return m_nID; }
		inline std::string GetArtist() const { return m_strArtist; }
		inline std::string GetAlbum() const { return m_strAlbum; }
		inline std::string GetTitle() const { return m_strTitle; }
		inline std::string GetTrack() const { return m_strTrack; }
		inline std::string GetTime() const { return m_strTime; }
		inline xb::Filename GetFilename() const { return m_strFilename; }
		inline unsigned int GetSeconds() const { return m_nSeconds; }
		inline bool IsSelected() const { return m_bSelected; }
		inline bool IsPlaying() const { return m_bPlaying; }

		inline void SetID(unsigned int nID) { m_nID = nID; }
		inline void SetArtist(const std::string& s) { m_strArtist = s; }
		inline void SetAlbum(const std::string& s) { m_strAlbum = s; }
		inline void SetTitle(const std::string& s) { m_strTitle = s; }
		inline void SetTrack(const std::string& s) { m_strTrack = s; }
		inline void SetTime(const std::string& s) { m_strTime = s; }
		inline void SetFilename(const xb::Filename& s) { m_strFilename = s; }
		inline void SetSeconds(unsigned int n) { m_nSeconds = n; }
		inline void SetSelected(bool bIn) { m_bSelected = bIn; }
		inline void SetPlaying(bool bIn) { m_bPlaying = bIn; }

		bool SatisfiesSearch(const std::string& strSearch) const;
		
		void Dump() const;

		friend std::ifstream& operator>>(std::ifstream& is, Song& song);
		friend std::ofstream& operator<<(std::ofstream& os, const Song& song);

		bool operator==(const Song& song) const;
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
