// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// SongList.cpp
//
// $Id: SongList.cpp,v 1.11 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "SongList.h"

#include <xb/ErrorManager.h>
#include <xb/FileUtil.h>
#include <xb/String.h>

#include "Util.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

SongList::SongList()
{

}

//------------------------------------------------------------------------------

SongList::~SongList()
{

}

//------------------------------------------------------------------------------

long
SongList::GetTotalSeconds() const
{
	long nSeconds = 0;
	SongList::const_iterator itr;
	for (itr = begin(); itr != end(); ++itr) {
		nSeconds += itr->GetSeconds();
	}

	return nSeconds;
}

//------------------------------------------------------------------------------

off_t
SongList::GetTotalFileSize() const
{
	off_t nResult = 0;
	SongList::const_iterator itr;
	for (itr = begin(); itr != end(); ++itr) {
		nResult += xb::FileUtil::GetSize(itr->GetFilename());
	}

	return nResult;
}

//------------------------------------------------------------------------------

void
SongList::Randomize()
{
	SongList listTemp(*this);

	vector<bool> vecUsed;
	vecUsed.reserve(listTemp.size());
	for (unsigned int i = 0; i < listTemp.size(); i++) {
		vecUsed[i] = false;
	}

	int nCount = 0;
	
	SongList::const_iterator itr;
	for (itr = listTemp.begin(); itr != listTemp.end(); ++itr) {
		int nRand;
		do {
			nRand = Util::Random(listTemp.size());
		} while (vecUsed[nRand]);

		(*this)[nRand] = *itr;
		vecUsed[nRand] = true;

		EM::I()->Trace() << "inserting song " << nCount++ << " at " <<
			nRand << endl;
	}

	std::string s("list size mismatch:  this = " +
				  xb::String(size()) + ", list = " +
				  xb::String(listTemp.size())); 
	Assert(size() == listTemp.size(), s.c_str());

}

//------------------------------------------------------------------------------

void
SongList::Dump() const
{
	SongList::const_iterator itr;
	for (itr = begin(); itr != end(); ++itr) {
		itr->Dump();
	}
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
