// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// SourceTable.h
//
// $Id: SourceTable.h,v 1.13 2002/03/12 00:04:51 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __SOURCETABLE__
#define __SOURCETABLE__

#include <xb/XCList.h>

#include <vector>
#include <xb/Pixmap.h>

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class SourceTable : public xb::XCList
{
	// Constants
	public:
		enum SourceType {
			LIBRARY,
			CD,
			PLAYLIST,
			NONE
		};
		
	// Types
	private:
		struct SourceInfo {
				std::string m_strName;
				SourceType m_nType;
		};
		
	// Variables
	private:
		int m_nLastRow;

		std::vector<SourceInfo> m_vecSourceInfo;

		int m_nDragRow;
		
		xb::Pixmap m_iconLibrary;
		xb::Pixmap m_iconCD;
		xb::Pixmap m_iconPlayList;

		static pthread_mutex_t s_ThreadLock;
		static pthread_mutex_t s_AddRowLock;

		std::string m_strCDArtist;
		std::string m_strCDTitle;
		
    // Methods
	public:
		SourceTable();
		virtual ~SourceTable();

		void UpdateSources(bool bInCallback);

		// PATRICK: AddRow(Source)
		void AddRow(SourceType nIcon, const std::string& strTitle);

		std::string GetName(unsigned int nRow) const;
		SourceType GetType(unsigned int nRow) const;

		virtual void clear();

		void UpdateCDInfo();
		
	private:
		// void AttachCell(Gtk::Widget& widget, int nRow, int nCol);
		
		void AddRow(const Gdk_Pixmap& pm, const Gdk_Bitmap& bm,
					const std::string& strTitle);


		void DNDReceive(GdkDragContext* pContext, gint x, gint y,
						GtkSelectionData* pData,
						guint info,
						guint32 time);

		int Motion(GdkEventMotion* pMotion);
		int DragMotion(GdkDragContext* pContext, gint x, gint y,
					   guint time);
		void DragLeave(GdkDragContext* pContext, guint time);

		void CDAvailableChange(bool bIn);

};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------

#endif

