// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// ThreadBurn.cpp
//
// $Id: ThreadBurn.cpp,v 1.6 2002/02/26 19:30:27 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "ThreadBurn.h"

#include <xb/ErrorManager.h>
#include <xb/Filename.h>
#include <xb/String.h>

#include "Controller.h"
#include "Output.h"
#include "Player.h"
#include "System.h"
#include "PropList.h"

//------------------------------------------------------------------------------

using xb::EM;

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

ThreadBurn::ThreadBurn(const SongList& listSongs, bool bNoAction) :
	Thread(),
	m_listSongs(listSongs),
	m_bNoAction(bNoAction)
{
}

//------------------------------------------------------------------------------

ThreadBurn::~ThreadBurn()
{
}

//------------------------------------------------------------------------------

void
ThreadBurn::Run()
{
	m_bActive = true;
	EM::I()->Trace() << "ThreadBurn::Run: starting burn process..." << endl;

	if (!m_bNoAction) {
		xb::Filename strBasePath = System::GetPropList()->GetValue("SpoolRoot");

	// clean directory

		std::string strTrackFilenames;
	
		int nTrack = 1;
		SongList::const_iterator itr;
		for (itr = m_listSongs.begin(); itr != m_listSongs.end(); ++itr) {
		
			std::string s = "Preparing track " + itr->GetTrack() + ": " +
				itr->GetTitle();
			EM::I()->Trace() << s << endl;
			StatusUpdate(s);

			EM::I()->Trace() << "mp3 filename: " << itr->GetFilename() << endl;
			std::string strWAVFilename = strBasePath + "track" + xb::String(nTrack) + ".wav";
			EM::I()->Trace() << "wav filename: " << strWAVFilename << endl;

			// decode
			/*
			DecoderWav decoder;
			decoder.Init();
			decoder.Run(itr->GetFilename(),	strWAVFilename);
			*/
			Player* pPlayer =
				Controller::GetPlayerPluginForFile(itr->GetFilename());
			if (pPlayer != NULL) {

				Output* pOutput = Controller::GetOutputPlugin("wav");
				AssertNotNull(pOutput);
				
				pPlayer->Init(pOutput, NULL);
				pPlayer->Play(itr->GetFilename(), nTrack);
				while (pPlayer->IsPlayComplete() == false) {
					Thread::usleep(10000);
				}

				pOutput->Close();
				
				delete pPlayer;
				
				strTrackFilenames += strWAVFilename + " ";
			}
			else {
				EM::I()->Warn() << "couldn't find input plugin for " <<
					itr->GetFilename() << endl;
			}
		
			nTrack++;
		}

		EM::I()->Trace() << "burning cd..." << endl;
		StatusUpdate("Burning CD...");
		std::string strCommand = System::GetPropList()->GetValue("CDRecord");
		std::string strArgs = "dev=" +
			System::GetPropList()->GetValue("CDRecordDevice") +
			" speed=" + System::GetPropList()->GetValue("BurnSpeed") +
			" -pad -silent -audio " + strTrackFilenames;

	/*
	int nPID;
	if ((nPID = fork()) == 0) {
		cout << "cdrecord command: " << strCommand << endl;
		cout << "cdrecord args: " << strArgs << endl;
		execl(strCommand.c_str(), strCommand.c_str(),
			  strArgs.c_str(), 0);
		_exit(1);
	}

	if (nPID > 0) {
		int nStatus;
		waitpid(nPID, &nStatus, 0);
	}
	*/
	
		std::string strFullCommand = strCommand + " " + strArgs;
		EM::I()->Noise() << "full command:  " << strCommand << endl;
		system(strFullCommand.c_str());
	}
	m_bActive = false;
	Done();
	EM::I()->Trace() << "Run: burn done." << endl;
}

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
