// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Util.cpp
//
// $Id: Util.cpp,v 1.8 2002/02/18 08:55:59 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#include "Util.h"

#include <stdio.h>
#include <errno.h>
#include <fcntl.h>
#include <linux/cdrom.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <time.h>
#include <unistd.h>
#include <xb/String.h>

//------------------------------------------------------------------------------

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

const long Util::s_nGiga = 1000000000;
const long Util::s_nMega = 1000000;
const long Util::s_nKilo = 1000;
int Util::s_nSeed = 0;

//------------------------------------------------------------------------------

std::string
Util::GetTimeStr(int nSecondsIn)
{
	std::string strResult;
	int nMinutes = nSecondsIn / 60;
	int nSeconds = nSecondsIn % 60;

	int nHours = nMinutes / 60;
	if (nHours > 0) {
		nMinutes -= nHours * 60;
		strResult = xb::String(nHours) + ":"; 
	}


	xb::String strMinutes(nMinutes);
	if (nHours > 0) {
		if (nMinutes < 10) {
			strMinutes.insert(0, "0", 1);
		}
	}
	xb::String strSeconds(nSeconds);
	if (nSeconds < 10) {
		strSeconds.insert(0, "0", 1);
	}

	strResult += strMinutes + ":" + strSeconds;

	return strResult;

}

//------------------------------------------------------------------------------

std::string
Util::GetTimeStr(int nMinutes, int nSeconds)
{
	return GetTimeStr(nMinutes * 60 + nSeconds);
}

//------------------------------------------------------------------------------

std::string
Util::GetFileSizeStr(off_t nFileSize)
{
	std::string strExt;
	long nDivisor;
	
	if (nFileSize > s_nGiga) {
		strExt = "GB";
		nDivisor = s_nGiga;
	}
	else if (nFileSize > s_nMega) {
		strExt = "MB";
		nDivisor = s_nMega;
	}
	else if (nFileSize > s_nKilo) {
		strExt = "KB";
		nDivisor = s_nKilo;
	}
	else {
		strExt = "B";
		nDivisor = 1;
	}

	int nPrimary = nFileSize / nDivisor;
	long nLeftover = nFileSize % nDivisor;
	int nLeftoverPercent = 100 * nLeftover / nDivisor;

	return (xb::String(nPrimary) + "." + xb::String(nLeftoverPercent) + strExt);
}

//------------------------------------------------------------------------------

int
Util::Random(int nMax)
{
	if (s_nSeed == 0) {
		s_nSeed = time(NULL) + getpid();
		srand(s_nSeed);
	}

	return rand() % nMax;
}

//------------------------------------------------------------------------------
/*
void
Util::Eject()
{
   int cdrom;                           

   if ((cdrom = open("/dev/cdrom",O_RDONLY)) <0) {
        perror("open");
        exit(1);
   }

   if (ioctl(cdrom,CDROMEJECT,0)<0) {
        perror("ioctl");
        exit(1);
   }
   
   close(cdrom);
}
*/
//------------------------------------------------------------------------------

NAMESPACE_CLOSE

//------------------------------------------------------------------------------
