// -*-c++-*-
//------------------------------------------------------------------------------
// xtunes - A comprehensive digital music system (http://xtunes.sourceforge.net)
// Copyright 2001,2002 Patrick Crosby <pcrosby@pobox.com>
//------------------------------------------------------------------------------
// Visualization.h
//
// $Id: Visualization.h,v 1.4 2002/02/17 01:03:50 pcrosby Exp $
//------------------------------------------------------------------------------
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
// details.
//
// You should have received a copy of the GNU General Public License along with
// this program; if not, write to the Free Software Foundation, Inc., 59 Temple
// Place, Suite 330, Boston, MA  02111-1307  USA
//------------------------------------------------------------------------------

#ifndef __Visualization__
#define __Visualization__

#include <cc++/thread.h>
#include <glib.h>
#include <vector>

#include "Output.h"

#include "Namespace.h"

NAMESPACE_OPEN

//------------------------------------------------------------------------------

class Visualization
{
	// Constants
	// Types
	private:
		struct Node {
				int nTime;
				int nChannels;
				int nLength;
				gint16 anData[2][512];
		};

		typedef vector<Node> DataList;
		
	// Variables
	private:
		DataList m_listData;
		ost::Mutex m_Mutex;

		gint16 m_anData[2][512];
		int m_nChannels;
		int m_nTime;
		
	// Methods
	public:
		Visualization();
		~Visualization();

		void SetData(int nTime, Output::AFormat nFormat, int nChannels,
					 int nLength, void* pData);

		const gint16* GetData() const;
		int GetChannels() const { return m_nChannels; }

		void Update(int nTime);

		void Reset();

	private:
		void ConvertData(Output::AFormat fmt, gpointer ptr, gint16* left,
						 gint16* right, gint nch, gint max);
		void FindData(int nTime, gint16* pData, int& nLength,
					  int& nChannels);
		
};

//------------------------------------------------------------------------------

NAMESPACE_CLOSE

#endif

//------------------------------------------------------------------------------
